import java.io.*;
/**
 * Clase estatica para el manejo de Consola
 * 
 * @author G. Cherencio
 * @version 1.0
 */
public class Consola {
    /**
     * muestro objeto por pantalla sin hacer \n
     * @param o objeto a imprimir
     */
    public static void print(Object o) { 
        System.out.print(o);
    }
    /**
     * muestro objeto por pantalla haciendo \n
     * @param o objeto a imprimir
     */
    public static void println(Object o) { 
        System.out.println(o);
    }
    /**
     * Ingreso linea por teclado sin prompt
     * @return retorno el texto ingresado por teclado
     */
    public static String readLine() {
        String linea = "";
        try {
            BufferedReader br = new BufferedReader(new InputStreamReader(System.in));
            linea = br.readLine();
        } catch(IOException ioe) {
            System.err.println("Consola.readLine(): Error, "+ioe.getMessage());
            return "";
        } finally {
            return linea;
        }
    }
    /**
     * Ingreso linea por teclado con prompt
     * @param prompt mensaje a enviar al usuario
     * @return retorno el texto ingresado por teclado
     */
    public static String readLine(String prompt) {
        print(prompt);
        return readLine();
    }
    /**
     * Ingreso linea por teclado con prompt y validacion
     * No permite el ingreso de valores distintos a los indicados
     * @param prompt mensaje a enviar al usuario
     * @param valores arreglo con los posibles valores a ingresar por teclado
     * @return retorno el texto ingresado por teclado
     */
    public static String readLine(String prompt,String[] valores) {
        String linea = "";
        do {
            linea = readLine(prompt);
        }while(!existePalabra(linea,valores));
        return  linea;
    }
    /**
     * Busca palabra dentro de valores y si se encuentra devuelve true sino false
     * @param palabra palabra a buscar
     * @param valores arreglo string en donde busco palabra
     * @return true si palabra es un elemento de valores o falso si no lo es
     */
    private static boolean existePalabra(String palabra,String[] valores) {
        for(String val : valores) {
            if ( palabra.equals(val) ) return true;
        }
        return false;
    }
    /**
     * Ingresa un entero por teclado
     */
    public static Integer readInt() {
        Integer entero = null;
        do {
            try {
                String linea = readLine();
                entero = new Integer(linea);
            } catch(NumberFormatException nfe) {
                entero = null;
            }
        } while(entero == null);
        return entero;
    }
    /**
     * Ingresa un entero por teclado
     * @param prompt prompt a mostrar al usuario
     */
    public static Integer readInt(String prompt) {
        Integer entero = null;
        do {
            try {
                String linea = readLine(prompt);
                entero = new Integer(linea);
            } catch(NumberFormatException nfe) {
                entero = null;
            }
        } while(entero == null);
        return entero;
    }
    /**
     * Ingresa un double por teclado
     */
    public static Double readDouble() {
        Double entero = null;
        do {
            try {
                String linea = readLine();
                entero = new Double(linea);
            } catch(NumberFormatException nfe) {
                entero = null;
            }
        } while(entero == null);
        return entero;
    }
    /**
     * Ingresa un double por teclado
     * @param prompt prompt a mostrar al usuario
     */
    public static Double readDouble(String prompt) {
        Double entero = null;
        do {
            try {
                String linea = readLine(prompt);
                entero = new Double(linea);
            } catch(NumberFormatException nfe) {
                entero = null;
            }
        } while(entero == null);
        return entero;
    }

}
