import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.table.*;
import java.util.*;
import java.sql.*;
/**
 * Clase Model para Producto a ser usada en JTable
 * 
 * @author G. Cherencio
 * @version 1.0
 */
public class ProductoModel extends AbstractTableModel implements KeyListener {
    private java.util.List<Producto> lst = new ArrayList<Producto>();
    private JTable tbl = null;

    /**
     * Constructor for objects of class ProductoModel
     */
    public ProductoModel()
    {
        // cargo los productos desde la base de datos
        try {
            FBCon f = FBCon.getFBCon(0);
            ResultSet rs = f.getResultSet("SELECT IDP,DESCR,PRECIO,STOCK FROM TBL_PRODUCTO ORDER BY DESCR ASC");
            while(rs.next()) {
                lst.add(new Producto(rs.getInt("idp"),rs.getString("descr"),rs.getDouble("precio"),rs.getInt("stock")));
            }
        } catch(SQLException sqle) {
            // Error recuperando clientes de la base de datos
            System.err.println("ProductoModel: Error, "+sqle.getMessage());
        }
    }
    // CUSTOM
    public void setJTable(JTable t) {
        tbl = t;
    }
    // METODOS QUE PODEMOS SOBRE-ESCRIBIR AbstractTableModel
    public Class<?> getColumnClass(int columnIndex) {
        switch(columnIndex) {
            case 0: // idp
                return Integer.class;
            case 1: // descr
                return String.class;
            case 2: // precio
                return Double.class;
            case 3: // stock
                return Integer.class;
        }
        return  Object.class;
    }
    public String getColumnName(int column) {
        switch(column) {
            case 0: // idp
                return "Codigo";
            case 1: // descr
                return "Descripcion";
            case 2: // precio
                return "Precio";
            case 3: // stock
                return "Stock";
        }
        return "";
    }
    public boolean isCellEditable(int rowIndex,int columnIndex) {
        if ( columnIndex == 0 ) return false; // no permito cambios en la columna IDP
        return true;
    }
    public void setValueAt(Object aValue,int rowIndex,int columnIndex) {
        Producto c = lst.get(rowIndex);
        switch(columnIndex) {
            case 1:
                if ( ((String) aValue).length() == 0 ) 
                    JOptionPane.showMessageDialog(null,"Debe Indicar Descripcion","Modifico Producto", 
                        JOptionPane.ERROR_MESSAGE);
                else c.setDescr((String) aValue);
                break;
            case 2: // precio
                if ( ((Double) aValue).doubleValue() <= 0 ) 
                    JOptionPane.showMessageDialog(null,"El Precio no puede ser Negativo o Cero","Modifico Producto", 
                        JOptionPane.ERROR_MESSAGE);
                else c.setPrecio((Double) aValue);
                break;
            case 3: // stock, permito tanto valores negativos como positivos o cero
                c.addStock(((Integer) aValue)-c.getStock());
                break;
            
        }
    }
    // METODOS ABSTRACTOS OBLIGATORIOS QUE DEBEMOS IMPLEMENTAR
    public int getRowCount() {
        return lst.size(); // la cantidad de elementos que tiene lst
    }
    public int getColumnCount() {
        return 4; // porque cliente tiene 4 atributos: idp,descr,precio,stock
    }
    public Object getValueAt(int row, int column) {
        Producto c = lst.get(row);
        switch(column) {
            case 0: // idp
                return c.getIdp();
            case 1: // descr
                return c.getDescr();
            case 2: // precio
                return c.getPrecio();
            case 3: // stock
                return c.getStock();
        }
        return  null;
    }
    // METODOS DE LA INTERFAZ KeyListener
    public void keyTyped(KeyEvent e) {
    }
    public void keyPressed(KeyEvent e) {
        if ( e.getKeyCode() == KeyEvent.VK_INSERT ) {
            System.out.println("presiono Insert!");
            String scodigo = null, sdescr = null, sprecio = null, sstock = null;
            boolean ok=false;
            do {
                try {
                    scodigo = JOptionPane.showInputDialog(null,"Codigo:","Nuevo Producto",JOptionPane.INFORMATION_MESSAGE);
                    int ncodigo = Integer.parseInt(scodigo);
                    if ( ncodigo < 0 ) continue;
                    // validacion adicional el codigo no debe estar repetido
                    boolean encontre = false;
                    for(Producto c : lst) {
                        if ( c.getIdp() == ncodigo ) {
                            encontre = true;
                            break;
                        }
                    }
                    if ( !encontre ) ok=true;
                    else JOptionPane.showMessageDialog(null,"El Codigo Producto " + scodigo + " ya existe!");
                } catch(NumberFormatException nfe) {
                }
            } while(!ok);
            do {
                sdescr  = JOptionPane.showInputDialog(null,"Descripcion:","Nuevo Producto",JOptionPane.INFORMATION_MESSAGE);
            } while(sdescr.length() == 0);
            ok=false;
            do {
                try {
                    sprecio  = JOptionPane.showInputDialog(null,"Precio:","Nuevo Producto",JOptionPane.INFORMATION_MESSAGE);
                    Double dd = Double.parseDouble(sprecio);
                    if ( dd.doubleValue() > 0.0 ) ok=true;
                } catch(NumberFormatException nfe) {
                }
            } while(!ok);
            ok=false;
            do {
                try {
                    sstock  = JOptionPane.showInputDialog(null,"Stock Inicial:","Nuevo Producto",JOptionPane.INFORMATION_MESSAGE);
                    Integer.parseInt(sstock);
                    ok=true;
                } catch(NumberFormatException nfe) {
                }
            } while(!ok);
            
            Producto c = new Producto(new Integer(scodigo),sdescr,new Double(sprecio),new Integer(sstock));
            // agrego nuevo producto a la lista
            lst.add(c);
            fireTableDataChanged(); // informo a la JTable que he cambiado los datos!
        }
        if ( e.getKeyCode() == KeyEvent.VK_DELETE ) {
            //System.out.println("presiono Delete!");
            if ( tbl != null ) {
                if ( tbl.getSelectedRow() != -1 ) {
                    // pedir confirmacion!
                    if ( JOptionPane.showOptionDialog(null,"¿Borro al Producto Seleccionado?",
                        "Borro Producto", JOptionPane.YES_NO_OPTION, JOptionPane.INFORMATION_MESSAGE,
                            null,new String[] { "Si", "No" } , "Si") == 0 ) {
                        lst.remove(tbl.getSelectedRow());
                        fireTableDataChanged(); // informo a la JTable que he cambiado los datos!
                    }
                } else System.out.println("Debe Seleccionar Producto a Borrar");
            } else System.out.println("Debe Indicar la JTable asociada a ProductoModel");
        }
    }
    public void keyReleased(KeyEvent e) {
    }

}
